@@ -42,8 +42,23 @@ from giscanner.sourcescanner import SourceScanner
 from giscanner.transformer import Transformer
 from . import utils
 
+def process_cflags_begin(option, opt, value, parser):
+    cflags = getattr(parser.values, option.dest)
+    while len(parser.rargs) > 0 and parser.rargs[0] != '--cflags-end':
+        cflags.append(parser.rargs.pop(0))
+
+def process_cflags_end(option, opt, value, parser):
+    pass
+
 def get_preprocessor_option_group(parser):
     group = optparse.OptionGroup(parser, "Preprocessor options")
+    group.add_option("", "--cflags-begin",
+                     help="Start preprocessor/compiler flags",
+                     dest="cflags", default=[],
+                     action="callback", callback=process_cflags_begin)
+    group.add_option("", "--cflags-end",
+                     help="End preprocessor/compiler flags",
+                     action="callback", callback=process_cflags_end)
     group.add_option("-I", help="Pre-processor include file",
                      action="append", dest="cpp_includes",
                      default=[])
@@ -158,6 +173,9 @@ match the namespace prefix.""")
     parser.add_option("", "--c-include",
                       action="append", dest="c_includes", default=[],
                       help="headers which should be included in C programs")
+    parser.add_option("", "--filelist",
+                      action="store", dest="filelist", default=[],
+                      help="file containing headers and sources to be scanned")
 
     group = get_preprocessor_option_group(parser)
     parser.add_option_group(group)
@@ -190,11 +208,7 @@ def passthrough_gir(path, f):
     parser = GIRParser()
     parser.parse(path)
 
-    writer = GIRWriter(parser.get_namespace(),
-                       parser.get_shared_libraries(),
-                       parser.get_includes(),
-                       parser.get_pkgconfig_packages(),
-                       parser.get_c_includes())
+    writer = GIRWriter(parser.get_namespace())
     f.write(writer.get_xml())
 
 def test_codegen(optstring):
@@ -252,6 +266,27 @@ def extract_filenames(args):
             filenames.append(os.path.abspath(arg))
     return filenames
 
+def extract_filelist(options):
+    filenames = []
+    if not os.path.exists(options.filelist):
+        _error('%s: no such filelist file' % (options.filelist, ))
+    filelist_file = open(options.filelist, "r")
+    lines = filelist_file.readlines()
+    for line in lines:
+        # We don't support real C++ parsing yet, but we should be able
+        # to understand C API implemented in C++ files.
+        filename = line.strip()
+        if (filename.endswith('.c') or filename.endswith('.cpp') or
+            filename.endswith('.cc') or filename.endswith('.cxx') or
+            filename.endswith('.h') or filename.endswith('.hpp') or
+            filename.endswith('.hxx')):
+            if not os.path.exists(filename):
+                _error('%s: Invalid filelist entry-no such file or directory' % (line, ))
+            # Make absolute, because we do comparisons inside scannerparser.c
+            # against the absolute path that cpp will give us
+            filenames.append(os.path.abspath(filename))
+    return filenames
+
 def create_namespace(options):
     if options.strip_prefix:
         print """g-ir-scanner: warning: Option --strip-prefix has been deprecated;
@@ -288,7 +323,6 @@ def create_transformer(namespace, options):
         transformer.disable_cache()
         transformer.set_passthrough_mode()
 
-    shown_include_warning = False
     for include in options.includes:
         if os.sep in include:
             _error("Invalid include path %r" % (include, ))
@@ -326,14 +360,18 @@ def create_binary(transformer, options, args):
     return shlibs
 
 def create_source_scanner(options, args):
-    filenames = extract_filenames(args)
+    if hasattr(options, 'filelist') and options.filelist:
+        filenames = extract_filelist(options)
+    else:
+        filenames = extract_filenames(args)
 
     # Run the preprocessor, tokenize and construct simple
     # objects representing the raw C symbols
     ss = SourceScanner()
     ss.set_cpp_options(options.cpp_includes,
                        options.cpp_defines,
-                       options.cpp_undefines)
+                       options.cpp_undefines,
+                       cflags=options.cflags)
     ss.parse_files(filenames)
     ss.parse_macros(filenames)
     return ss
@@ -383,8 +421,9 @@ def scanner_main(args):
     if options.test_codegen:
         return test_codegen(options.test_codegen)
 
-    if len(args) <= 1:
-        _error('Need at least one filename')
+    if hasattr(options, 'filelist') and not options.filelist:
+        if len(args) <= 1:
+            _error('Need at least one filename')
 
     if not options.namespace_name:
         _error('Namespace name missing')
@@ -426,6 +465,8 @@ def scanner_main(args):
     else:
         shlibs = []
 
+    transformer.namespace.shared_libraries = shlibs
+
     main = MainTransformer(transformer, blocks)
     main.transform()
 
@@ -448,8 +489,9 @@ def scanner_main(args):
     else:
         exported_packages = options.packages
 
-    writer = Writer(transformer.namespace, shlibs, transformer.get_includes(),
-                    exported_packages, options.c_includes)
+    transformer.namespace.c_includes = options.c_includes
+    transformer.namespace.exported_packages = exported_packages
+    writer = Writer(transformer.namespace)
     data = writer.get_xml()
 
     write_output(data, options)
